package java.time.temporal;

import java.time.Duration;
import java.util.ResourceBundle;
import sun.util.locale.provider.LocaleProviderAdapter;
import java.util.Objects;
import java.util.Locale;
import java.time.chrono.ChronoLocalDate;
import java.time.format.ResolverStyle;
import java.util.Map;
import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.DateTimeException;
import java.time.chrono.IsoChronology;
import java.time.chrono.Chronology;

public final class IsoFields
{
    public static final TemporalField DAY_OF_QUARTER;
    public static final TemporalField QUARTER_OF_YEAR;
    public static final TemporalField WEEK_OF_WEEK_BASED_YEAR;
    public static final TemporalField WEEK_BASED_YEAR;
    public static final TemporalUnit WEEK_BASED_YEARS;
    public static final TemporalUnit QUARTER_YEARS;
    
    private IsoFields() {
        throw new AssertionError("Not instantiable");
    }
    
    static {
        DAY_OF_QUARTER = IsoFields.Field.DAY_OF_QUARTER;
        QUARTER_OF_YEAR = IsoFields.Field.QUARTER_OF_YEAR;
        WEEK_OF_WEEK_BASED_YEAR = IsoFields.Field.WEEK_OF_WEEK_BASED_YEAR;
        WEEK_BASED_YEAR = IsoFields.Field.WEEK_BASED_YEAR;
        WEEK_BASED_YEARS = IsoFields.Unit.WEEK_BASED_YEARS;
        QUARTER_YEARS = IsoFields.Unit.QUARTER_YEARS;
    }
    
    private enum Field implements TemporalField
    {
        DAY_OF_QUARTER {
            @Override
            public TemporalUnit getBaseUnit() {
                return ChronoUnit.DAYS;
            }
            
            @Override
            public TemporalUnit getRangeUnit() {
                return IsoFields.QUARTER_YEARS;
            }
            
            @Override
            public ValueRange range() {
                return ValueRange.of(1L, 90L, 92L);
            }
            
            @Override
            public boolean isSupportedBy(TemporalAccessor temporalAccessor) {
                return temporalAccessor.isSupported(ChronoField.DAY_OF_YEAR) && temporalAccessor.isSupported(ChronoField.MONTH_OF_YEAR) && temporalAccessor.isSupported(ChronoField.YEAR) && IsoFields.Field.access$100(temporalAccessor);
            }
            
            @Override
            public ValueRange rangeRefinedBy(TemporalAccessor temporalAccessor) {
                if (!this.isSupportedBy(temporalAccessor)) {
                    throw new UnsupportedTemporalTypeException("Unsupported field: DayOfQuarter");
                }
                long long1 = temporalAccessor.getLong(IsoFields.Field.1.QUARTER_OF_YEAR);
                if (long1 == 1L) {
                    return IsoChronology.INSTANCE.isLeapYear(temporalAccessor.getLong(ChronoField.YEAR)) ? ValueRange.of(1L, 91L) : ValueRange.of(1L, 90L);
                }
                if (long1 == 2L) {
                    return ValueRange.of(1L, 91L);
                }
                if (long1 == 3L || long1 == 4L) {
                    return ValueRange.of(1L, 92L);
                }
                return this.range();
            }
            
            @Override
            public long getFrom(TemporalAccessor temporalAccessor) {
                if (!this.isSupportedBy(temporalAccessor)) {
                    throw new UnsupportedTemporalTypeException("Unsupported field: DayOfQuarter");
                }
                return (long)(temporalAccessor.get(ChronoField.DAY_OF_YEAR) - IsoFields.Field.QUARTER_DAYS[(temporalAccessor.get(ChronoField.MONTH_OF_YEAR) - 1) / 3 + (IsoChronology.INSTANCE.isLeapYear(temporalAccessor.getLong(ChronoField.YEAR)) ? 4 : 0)]);
            }
            
            @Override
            public <R extends Temporal> R adjustInto(R r, long n) {
                long from = this.getFrom(r);
                this.range().checkValidValue(n, this);
                return r.with(ChronoField.DAY_OF_YEAR, r.getLong(ChronoField.DAY_OF_YEAR) + (n - from));
            }
            
            @Override
            public ChronoLocalDate resolve(Map<TemporalField, Long> map, TemporalAccessor temporalAccessor, ResolverStyle resolverStyle) {
                Long n = (Long)map.get(ChronoField.YEAR);
                Long n2 = (Long)map.get(IsoFields.Field.1.QUARTER_OF_YEAR);
                if (n == null || n2 == null) {
                    return null;
                }
                int checkValidIntValue = ChronoField.YEAR.checkValidIntValue((long)n);
                long longValue = (long)map.get(IsoFields.Field.1.DAY_OF_QUARTER);
                IsoFields.Field.access$300(temporalAccessor);
                LocalDate localDate;
                long subtractExact;
                if (resolverStyle == ResolverStyle.LENIENT) {
                    localDate = LocalDate.of(checkValidIntValue, 1, 1).plusMonths(Math.multiplyExact(Math.subtractExact((long)n2, 1L), 3L));
                    subtractExact = Math.subtractExact(longValue, 1L);
                }
                else {
                    localDate = LocalDate.of(checkValidIntValue, (IsoFields.Field.1.QUARTER_OF_YEAR.range().checkValidIntValue((long)n2, IsoFields.Field.1.QUARTER_OF_YEAR) - 1) * 3 + 1, 1);
                    if (longValue < 1L || longValue > 90L) {
                        if (resolverStyle == ResolverStyle.STRICT) {
                            this.rangeRefinedBy(localDate).checkValidValue(longValue, this);
                        }
                        else {
                            this.range().checkValidValue(longValue, this);
                        }
                    }
                    subtractExact = longValue - 1L;
                }
                map.remove(this);
                map.remove(ChronoField.YEAR);
                map.remove(IsoFields.Field.1.QUARTER_OF_YEAR);
                return localDate.plusDays(subtractExact);
            }
            
            @Override
            public String toString() {
                return "DayOfQuarter";
            }
        }, 
        QUARTER_OF_YEAR {
            @Override
            public TemporalUnit getBaseUnit() {
                return IsoFields.QUARTER_YEARS;
            }
            
            @Override
            public TemporalUnit getRangeUnit() {
                return ChronoUnit.YEARS;
            }
            
            @Override
            public ValueRange range() {
                return ValueRange.of(1L, 4L);
            }
            
            @Override
            public boolean isSupportedBy(TemporalAccessor temporalAccessor) {
                return temporalAccessor.isSupported(ChronoField.MONTH_OF_YEAR) && IsoFields.Field.access$100(temporalAccessor);
            }
            
            @Override
            public long getFrom(TemporalAccessor temporalAccessor) {
                if (!this.isSupportedBy(temporalAccessor)) {
                    throw new UnsupportedTemporalTypeException("Unsupported field: QuarterOfYear");
                }
                return (temporalAccessor.getLong(ChronoField.MONTH_OF_YEAR) + 2L) / 3L;
            }
            
            @Override
            public <R extends Temporal> R adjustInto(R r, long n) {
                long from = this.getFrom(r);
                this.range().checkValidValue(n, this);
                return r.with(ChronoField.MONTH_OF_YEAR, r.getLong(ChronoField.MONTH_OF_YEAR) + (n - from) * 3L);
            }
            
            @Override
            public String toString() {
                return "QuarterOfYear";
            }
        }, 
        WEEK_OF_WEEK_BASED_YEAR {
            @Override
            public String getDisplayName(Locale obj) {
                Objects.requireNonNull(obj, "locale");
                ResourceBundle javaTimeFormatData = LocaleProviderAdapter.getResourceBundleBased().getLocaleResources(obj).getJavaTimeFormatData();
                return javaTimeFormatData.containsKey("field.week") ? javaTimeFormatData.getString("field.week") : this.toString();
            }
            
            @Override
            public TemporalUnit getBaseUnit() {
                return ChronoUnit.WEEKS;
            }
            
            @Override
            public TemporalUnit getRangeUnit() {
                return IsoFields.WEEK_BASED_YEARS;
            }
            
            @Override
            public ValueRange range() {
                return ValueRange.of(1L, 52L, 53L);
            }
            
            @Override
            public boolean isSupportedBy(TemporalAccessor temporalAccessor) {
                return temporalAccessor.isSupported(ChronoField.EPOCH_DAY) && IsoFields.Field.access$100(temporalAccessor);
            }
            
            @Override
            public ValueRange rangeRefinedBy(TemporalAccessor temporal) {
                if (!this.isSupportedBy(temporal)) {
                    throw new UnsupportedTemporalTypeException("Unsupported field: WeekOfWeekBasedYear");
                }
                return IsoFields.Field.access$400(LocalDate.from(temporal));
            }
            
            @Override
            public long getFrom(TemporalAccessor temporal) {
                if (!this.isSupportedBy(temporal)) {
                    throw new UnsupportedTemporalTypeException("Unsupported field: WeekOfWeekBasedYear");
                }
                return (long)IsoFields.Field.access$500(LocalDate.from(temporal));
            }
            
            @Override
            public <R extends Temporal> R adjustInto(R r, long x) {
                this.range().checkValidValue(x, this);
                return r.plus(Math.subtractExact(x, this.getFrom(r)), ChronoUnit.WEEKS);
            }
            
            @Override
            public ChronoLocalDate resolve(Map<TemporalField, Long> map, TemporalAccessor temporalAccessor, ResolverStyle resolverStyle) {
                Long n = (Long)map.get(IsoFields.Field.3.WEEK_BASED_YEAR);
                Long n2 = (Long)map.get(ChronoField.DAY_OF_WEEK);
                if (n == null || n2 == null) {
                    return null;
                }
                int checkValidIntValue = IsoFields.Field.3.WEEK_BASED_YEAR.range().checkValidIntValue((long)n, IsoFields.Field.3.WEEK_BASED_YEAR);
                long longValue = (long)map.get(IsoFields.Field.3.WEEK_OF_WEEK_BASED_YEAR);
                IsoFields.Field.access$300(temporalAccessor);
                LocalDate localDate = LocalDate.of(checkValidIntValue, 1, 4);
                LocalDate localDate2;
                if (resolverStyle == ResolverStyle.LENIENT) {
                    long longValue2 = (long)n2;
                    if (longValue2 > 7L) {
                        localDate = localDate.plusWeeks((longValue2 - 1L) / 7L);
                        longValue2 = (longValue2 - 1L) % 7L + 1L;
                    }
                    else if (longValue2 < 1L) {
                        localDate = localDate.plusWeeks(Math.subtractExact(longValue2, 7L) / 7L);
                        longValue2 = (longValue2 + 6L) % 7L + 1L;
                    }
                    localDate2 = localDate.plusWeeks(Math.subtractExact(longValue, 1L)).with(ChronoField.DAY_OF_WEEK, longValue2);
                }
                else {
                    int checkValidIntValue2 = ChronoField.DAY_OF_WEEK.checkValidIntValue((long)n2);
                    if (longValue < 1L || longValue > 52L) {
                        if (resolverStyle == ResolverStyle.STRICT) {
                            IsoFields.Field.access$400(localDate).checkValidValue(longValue, this);
                        }
                        else {
                            this.range().checkValidValue(longValue, this);
                        }
                    }
                    localDate2 = localDate.plusWeeks(longValue - 1L).with(ChronoField.DAY_OF_WEEK, (long)checkValidIntValue2);
                }
                map.remove(this);
                map.remove(IsoFields.Field.3.WEEK_BASED_YEAR);
                map.remove(ChronoField.DAY_OF_WEEK);
                return localDate2;
            }
            
            @Override
            public String toString() {
                return "WeekOfWeekBasedYear";
            }
        }, 
        WEEK_BASED_YEAR {
            @Override
            public TemporalUnit getBaseUnit() {
                return IsoFields.WEEK_BASED_YEARS;
            }
            
            @Override
            public TemporalUnit getRangeUnit() {
                return ChronoUnit.FOREVER;
            }
            
            @Override
            public ValueRange range() {
                return ChronoField.YEAR.range();
            }
            
            @Override
            public boolean isSupportedBy(TemporalAccessor temporalAccessor) {
                return temporalAccessor.isSupported(ChronoField.EPOCH_DAY) && IsoFields.Field.access$100(temporalAccessor);
            }
            
            @Override
            public long getFrom(TemporalAccessor temporal) {
                if (!this.isSupportedBy(temporal)) {
                    throw new UnsupportedTemporalTypeException("Unsupported field: WeekBasedYear");
                }
                return (long)IsoFields.Field.access$600(LocalDate.from(temporal));
            }
            
            @Override
            public <R extends Temporal> R adjustInto(R temporal, long n) {
                if (!this.isSupportedBy(temporal)) {
                    throw new UnsupportedTemporalTypeException("Unsupported field: WeekBasedYear");
                }
                int checkValidIntValue = this.range().checkValidIntValue(n, IsoFields.Field.4.WEEK_BASED_YEAR);
                LocalDate from = LocalDate.from(temporal);
                int value = from.get(ChronoField.DAY_OF_WEEK);
                int access$500 = IsoFields.Field.access$500(from);
                if (access$500 == 53 && IsoFields.Field.access$700(checkValidIntValue) == 52) {
                    access$500 = 52;
                }
                LocalDate of = LocalDate.of(checkValidIntValue, 1, 4);
                return temporal.with(of.plusDays((long)(value - of.get(ChronoField.DAY_OF_WEEK) + (access$500 - 1) * 7)));
            }
            
            @Override
            public String toString() {
                return "WeekBasedYear";
            }
        };
        
        private static final int[] QUARTER_DAYS;
        
        @Override
        public boolean isDateBased() {
            return true;
        }
        
        @Override
        public boolean isTimeBased() {
            return false;
        }
        
        @Override
        public ValueRange rangeRefinedBy(TemporalAccessor temporalAccessor) {
            return this.range();
        }
        
        private static boolean isIso(TemporalAccessor temporal) {
            return Chronology.from(temporal).equals(IsoChronology.INSTANCE);
        }
        
        private static void ensureIso(TemporalAccessor temporalAccessor) {
            if (!isIso(temporalAccessor)) {
                throw new DateTimeException("Resolve requires IsoChronology");
            }
            return;
        }
        
        private static ValueRange getWeekRange(LocalDate localDate) {
            return ValueRange.of(1L, (long)getWeekRange(getWeekBasedYear(localDate)));
        }
        
        private static int getWeekRange(int year) {
            LocalDate of = LocalDate.of(year, 1, 1);
            if (of.getDayOfWeek() == DayOfWeek.THURSDAY || (of.getDayOfWeek() == DayOfWeek.WEDNESDAY && of.isLeapYear())) {
                return 53;
            }
            return 52;
        }
        
        private static int getWeek(LocalDate localDate) {
            int ordinal = localDate.getDayOfWeek().ordinal();
            int n = localDate.getDayOfYear() - 1;
            int n2 = n + (3 - ordinal);
            int n3 = n2 - n2 / 7 * 7 - 3;
            if (n3 < -3) {
                n3 += 7;
            }
            if (n < n3) {
                return (int)getWeekRange(localDate.withDayOfYear(180).minusYears(1L)).getMaximum();
            }
            int n4 = (n - n3) / 7 + 1;
            if (n4 == 53 && n3 != -3 && (n3 != -2 || !localDate.isLeapYear())) {
                n4 = 1;
            }
            return n4;
        }
        
        private static int getWeekBasedYear(LocalDate localDate) {
            int year = localDate.getYear();
            int dayOfYear = localDate.getDayOfYear();
            if (dayOfYear <= 3) {
                if (dayOfYear - localDate.getDayOfWeek().ordinal() < -2) {
                    --year;
                }
            }
            else if (dayOfYear >= 363 && dayOfYear - 363 - (localDate.isLeapYear() ? 1 : 0) - localDate.getDayOfWeek().ordinal() >= 0) {
                ++year;
            }
            return year;
        }
        
        static boolean access$100(TemporalAccessor temporalAccessor) {
            return isIso(temporalAccessor);
        }
        
        static void access$300(TemporalAccessor temporalAccessor) {
            ensureIso(temporalAccessor);
        }
        
        static ValueRange access$400(LocalDate localDate) {
            return getWeekRange(localDate);
        }
        
        static int access$500(LocalDate localDate) {
            return getWeek(localDate);
        }
        
        static int access$600(LocalDate localDate) {
            return getWeekBasedYear(localDate);
        }
        
        static int access$700(int n) {
            return getWeekRange(n);
        }
        
        static {
            QUARTER_DAYS = new int[] { 0, 90, 181, 273, 0, 91, 182, 274 };
        }
    }
    
    private enum Unit implements TemporalUnit
    {
        WEEK_BASED_YEARS("WeekBasedYears", Duration.ofSeconds(31556952L)), 
        QUARTER_YEARS("QuarterYears", Duration.ofSeconds(7889238L));
        
        private final String name;
        private final Duration duration;
        
        private Unit(String name2, Duration duration) {
            this.name = name2;
            this.duration = duration;
        }
        
        @Override
        public Duration getDuration() {
            return this.duration;
        }
        
        @Override
        public boolean isDurationEstimated() {
            return true;
        }
        
        @Override
        public boolean isDateBased() {
            return true;
        }
        
        @Override
        public boolean isTimeBased() {
            return false;
        }
        
        @Override
        public boolean isSupportedBy(Temporal temporal) {
            return temporal.isSupported(ChronoField.EPOCH_DAY);
        }
        
        @Override
        public <R extends Temporal> R addTo(R r, long y) {
            switch (this) {
                case WEEK_BASED_YEARS: {
                    return r.with(IsoFields.WEEK_BASED_YEAR, Math.addExact((long)r.get(IsoFields.WEEK_BASED_YEAR), y));
                }
                case QUARTER_YEARS: {
                    return r.plus(y / 4L, ChronoUnit.YEARS).plus(y % 4L * 3L, ChronoUnit.MONTHS);
                }
                default: {
                    throw new IllegalStateException("Unreachable");
                }
            }
        }
        
        @Override
        public long between(Temporal temporal, Temporal temporal2) {
            if (temporal.getClass() != temporal2.getClass()) {
                return temporal.until(temporal2, this);
            }
            switch (this) {
                case WEEK_BASED_YEARS: {
                    return Math.subtractExact(temporal2.getLong(IsoFields.WEEK_BASED_YEAR), temporal.getLong(IsoFields.WEEK_BASED_YEAR));
                }
                case QUARTER_YEARS: {
                    return temporal.until(temporal2, ChronoUnit.MONTHS) / 3L;
                }
                default: {
                    throw new IllegalStateException("Unreachable");
                }
            }
        }
        
        @Override
        public String toString() {
            return this.name;
        }
    }
}
